import tkinter as tk, os, json, time
from tkinter import filedialog

W, H, G = 800, 600, 10
R, C = H // G, W // G
E, S, Wt, Wa, F = range(5)
Cs = {S: "yellow", Wt: "blue", Wa: "gray", F: "orange"}
Bcs = {S: "#FFFF99", Wt: "#99CCFF", Wa: "#CCCCCC", F: "#FFCC99"}
grid, cur, dm, lt, fps = [[E] * C for _ in range(R)], S, False, time.time(), 0

def draw(canvas):
    canvas.delete("all")
    canvas.configure(bg="black" if dm else "white")
    for row in range(R):
        for col in range(C):
            color = Cs.get(grid[row][col])
            if color:
                x1, y1 = col * G, row * G
                x2, y2 = x1 + G, y1 + G
                canvas.create_rectangle(x1, y1, x2, y2, fill=color, outline="")
    canvas.create_text(10, 10, anchor=tk.NW, text=f"FPS: {fps:.2f}", fill="white" if dm else "black")

def update():
    global lt, fps
    fps, lt = 1 / (time.time() - lt), time.time()
    for row in range(R - 2, -1, -1):
        for col in range(C):
            if grid[row][col] == S and grid[row + 1][col] in [E, Wt]:
                grid[row][col], grid[row + 1][col] = (Wt, S) if grid[row + 1][col] == Wt else (E, S)
            elif grid[row][col] == Wt:
                for dx, dy in [(0, 1), (-1, 1), (1, 1), (-1, 0), (1, 0)]:
                    if 0 <= col + dx < C and 0 <= row + dy < R and grid[row + dy][col + dx] == E:
                        grid[row][col], grid[row + dy][col + dx] = E, Wt
                        break
            elif grid[row][col] == F:
                for dx, dy in [(0, -1), (-1, -1), (1, -1)]:
                    if 0 <= col + dx < C and 0 <= row + dy < R and grid[row + dy][col + dx] == E:
                        grid[row][col], grid[row + dy][col + dx] = E, F
                        break

def add(x, y):
    col, row = x // G, y // G
    if grid[row][col] == E or cur == E:
        grid[row][col] = cur

def clear():
    global grid
    grid = [[E] * C for _ in range(R)]

def toggle_dm():
    global dm
    dm = not dm
    root.configure(bg="black" if dm else "white")

def on_mouse(event, down):
    if down:
        add(event.x, event.y)

def set_elem(element):
    global cur
    cur = element

def import_elements():
    path = filedialog.askopenfilename(title="Import Element", filetypes=[("JSON Files", "*.json")])
    if path:
        with open(path, 'r') as f:
            new_elem = json.load(f)
            elem_id = max(Cs.keys()) + 1
            Cs[elem_id] = new_elem['color']
            Bcs[elem_id] = new_elem['button_color']
            buttons.append((new_elem['name'], elem_id))
            update_buttons()

def loop():
    update()
    draw(canvas)
    root.after(50, loop)

def update_buttons():
    for widget in frame.winfo_children():
        widget.destroy()
    for text, element in buttons:
        cmd = element if text in ["Clear", "Dark"] else lambda e=element: set_elem(e)
        tk.Button(frame, text=text, command=cmd, width=7, bg=Bcs.get(element, "white")).pack(side=tk.LEFT, pady=5)
    tk.Button(frame, text="Import", command=import_elements, width=7).pack(side=tk.LEFT, pady=5)
    tk.Button(frame, text="Pause", command=pause_simulation, width=7).pack(side=tk.LEFT, pady=5)

def pause_simulation():
    global paused
    paused = not paused

root = tk.Tk()
root.title("The Blocky Toy DEV")
canvas = tk.Canvas(root, width=W, height=H)
canvas.pack(side=tk.TOP)
canvas.bind("<Button-1>", lambda e: on_mouse(e, True))
canvas.bind("<B1-Motion>", lambda e: on_mouse(e, True))
canvas.bind("<ButtonRelease-1>", lambda e: on_mouse(e, False))
root.bind("<Key>", lambda e: toggle_dm() if e.char == 'd' else None)
root.bind("<q>", lambda e: root.quit())

frame = tk.Frame(root, bd=2, relief=tk.SUNKEN)
frame.pack(side=tk.BOTTOM, fill=tk.X, padx=10, pady=10)

buttons = [("Sand", S), ("Water", Wt), ("Wall", Wa), ("Fire", F), ("Erase", E), ("Clear", clear), ("Dark", toggle_dm)]
update_buttons()

root.after(50, loop)
root.mainloop()
